-- the vehicle to test with
local physicsFPS  = 2000
local physicsSteps = 80 -- max physics steps do *NOT* increase beyond this point
local gfxSteps   = 15
local currentObjects = 0

-- lets not slow down due to debug
setLogLevel(LOG_ERROR)

function myBenchStep(testVehicle, n)
    -- delete everything
    -- spawn n vehicles
    for i=currentObjects + 1,n do
        BeamEngine:spawnObject(testVehicle, float3(3 * n, 3 * n, 3 * n)) -- this assumes that the object is max. 3 meters high
        currentObjects = currentObjects + 1
    end
    
    -- run the update loop and measure the total time

    local dt = (physicsSteps + 0.1)/ physicsFPS    
    local hp = HighPerfTimer()    
    for i=1,gfxSteps
    do
        BeamEngine:update(dt)
    end
    return hp:stop()
end

function myBenchPhysics(vehicleMin, vehicleMax, testVehicle, filename)
    local maxMbeams = 0
    local result = {}

    -- init
    BeamEngine:deleteAllObjects()
    BeamEngine:spawnObject(testVehicle, float3(0,3,0))
    currentObjects = 1
    local nodecount = vehicleLua(0, "#v.data.nodes")
    local beamcount = vehicleLua(0, "#v.data.beams")
    
    -- test start
    for n=vehicleMin,vehicleMax,1
    do
        -- do a warmup round
        myBenchStep(testVehicle, n)
        local totalTime = 0
        totalTime = totalTime + myBenchStep(testVehicle, n)

        totalTime = totalTime + myBenchStep(testVehicle, n)
        local totalSteps = physicsSteps * 2 * gfxSteps
        local totalBeams = totalSteps * beamcount * n
        local totalNodes = totalSteps * nodecount * n
        
        -- then calc the stats and output them
        local nodespersec = totalNodes / totalTime
        local beamspersec = totalBeams / totalTime
        local realtime    = (100 * (1000/physicsFPS) * totalSteps)/ totalTime
        maxMbeams = math.max(maxMbeams, beamspersec)
        if n == 0 then
            nodespersec = 0
            beamspersec = 0
        end

        result[n] = { vehicleCount = n, beamspersec = beamspersec, nodespersec = nodespersec, realtime = realtime}
    end
    
    serializeJsonToFile(filename, result)
end

myBenchPhysics(1, 10, 'vehicles/pickup/', 'bench-result-pickup.json')
myBenchPhysics(1, 5, 'vehicles/semi/', 'bench-result-semi.json')

